@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////////////////////
 //                           **** WAVPACK ****                            //
 //                  Hybrid Lossless Wavefile Compressor                   //
-//                Copyright (c) 1998 - 2016 David Bryant.                 //
+//                Copyright (c) 1998 - 2017 David Bryant.                 //
 //                          All Rights Reserved.                          //
 //      Distributed under the BSD Software License (see license.txt)      //
 ////////////////////////////////////////////////////////////////////////////
@@ -61,7 +61,7 @@
 
 static const char *sign_on = "\n"
 " WVGAIN  ReplayGain Scanner/Tagger for WavPack  %s Version %s\n"
-" Copyright (c) 2005 - 2016 David Bryant.  All Rights Reserved.\n\n";
+" Copyright (c) 2005 - 2017 David Bryant.  All Rights Reserved.\n\n";
 
 static const char *version_warning = "\n"
 " WARNING: WVGAIN using libwavpack version %s, expected %s (see README)\n\n";
@@ -1430,7 +1430,15 @@ static int WideCharToUTF8 (const wchar_t *Wide, unsigned char *pUTF8, int len)
 
 static void TextToUTF8 (void *string, int len)
 {
-    if (* (wchar_t *) string == 0xFEFF) {
+    unsigned char *inp = string;
+
+    // simple case: test for UTF8 BOM and if so, simply delete the BOM
+
+    if (len > 3 && inp [0] == 0xEF && inp [1] == 0xBB && inp [2] == 0xBF) {
+        memmove (inp, inp + 3, len - 3);
+        inp [len - 3] = 0;
+    }
+    else if (* (wchar_t *) string == 0xFEFF) {
         wchar_t *temp = _wcsdup (string);
 
         WideCharToUTF8 (temp + 1, (unsigned char *) string, len);
