@@ -104,7 +104,7 @@ static void mix_samples_with_gain (float *destin, float *source, int num_samples
 static void truncate_float_samples (float *samples, int num_samples, int bits);
 static void float_to_integer_samples (float *samples, int num_samples, int bits);
 static void float_to_32bit_integer_samples (float *samples, int num_samples);
-static unsigned char *format_samples (int bps, unsigned char *dst, int32_t *src, uint32_t samcnt);
+static void *store_samples (void *dst, int32_t *src, int qmode, int bps, int count);
 static double frandom (void);
 
 typedef struct {
@@ -291,11 +291,11 @@ done:
 static int seeking_test (char *filename, uint32_t test_count)
 {
     char error [80];
-    WavpackContext *wpc = WavpackOpenFileInput (filename, error, OPEN_WVC, 0);
-    uint32_t min_chunk_size = 256, total_samples, sample_count = 0;
+    WavpackContext *wpc = WavpackOpenFileInput (filename, error, OPEN_WVC | OPEN_DSD_NATIVE | OPEN_ALT_TYPES, 0);
+    int64_t min_chunk_size = 256, total_samples, sample_count = 0;
     char md5_string1 [] = "????????????????????????????????";
     char md5_string2 [] = "????????????????????????????????";
-    int32_t *decoded_samples, num_chans, bps, test_index;
+    int32_t *decoded_samples, num_chans, bps, test_index, qmode;
     unsigned char md5_initial [16], md5_stored [16];
     MD5_CTX md5_global, md5_local;
     unsigned char *chunked_md5;
@@ -309,14 +309,20 @@ static int seeking_test (char *filename, uint32_t test_count)
     }
 
     num_chans = WavpackGetNumChannels (wpc);
-    total_samples = WavpackGetNumSamples (wpc);
+    total_samples = WavpackGetNumSamples64 (wpc);
     bps = WavpackGetBytesPerSample (wpc);
+    qmode = WavpackGetQualifyMode (wpc);
 
-    if (total_samples < 2 || total_samples == (uint32_t) -1) {
+    if (total_samples < 2 || total_samples == -1) {
         printf ("seeking_test(): can't determine file size!\n");
         return -1;
     }
 
+    if (qmode & QMODE_DSD_IN_BLOCKS) {
+        printf ("seeking_test(): can't handle blocked DSD audio (i.e., from .dsf files)!\n");
+        return -1;
+    }
+
     // For very short files, reduce the minimum chunk size
 
     while (min_chunk_size > 1 && total_samples / min_chunk_size < 256)
@@ -330,7 +336,7 @@ static int seeking_test (char *filename, uint32_t test_count)
         decoded_samples = malloc (sizeof (int32_t) * chunk_samples * num_chans);
         chunked_md5 = malloc (total_chunks * 16);
 
-        if (!total_chunks || !decoded_samples) {
+        if (!chunked_md5 || !decoded_samples) {
             printf ("seeking_test(): can't allocate memory!\n");
             return -1;
         }
@@ -346,7 +352,7 @@ static int seeking_test (char *filename, uint32_t test_count)
             if (!samples)
                 break;
 
-            format_samples (bps, (unsigned char *) decoded_samples, decoded_samples, samples * num_chans);
+            store_samples (decoded_samples, decoded_samples, qmode, bps, samples * num_chans);
             MD5Update (&md5_global, (unsigned char *) decoded_samples, bps * samples * num_chans);
 
             MD5Init (&md5_local);
@@ -386,7 +392,7 @@ static int seeking_test (char *filename, uint32_t test_count)
                 sprintf (md5_string2 + (i * 2), "%02x", md5_initial [i]);
             }
 
-            printf ("stored/actual sample count: %u / %u\n", total_samples, sample_count);
+            printf ("stored/actual sample count: %lld / %lld\n", (long long int) total_samples, (long long int) sample_count);
             if (file_has_md5) printf ("stored md5: %s\n", md5_string1);
             printf ("actual md5: %s\n", md5_string2);
 
@@ -412,7 +418,7 @@ static int seeking_test (char *filename, uint32_t test_count)
 
         if (frandom() < 0.5) {
             WavpackCloseFile (wpc);
-            wpc = WavpackOpenFileInput (filename, error, OPEN_WVC, 0);
+            wpc = WavpackOpenFileInput (filename, error, OPEN_WVC | OPEN_DSD_NATIVE | OPEN_ALT_TYPES, 0);
 
             if (!wpc) {
                 printf ("seeking_test(): error \"%s\" reopening input file \"%s\"\n", error, filename);
@@ -439,7 +445,7 @@ static int seeking_test (char *filename, uint32_t test_count)
 
             stop_chunk = start_chunk + num_chunks - 1;
 
-            if (!WavpackSeekSample (wpc, start_chunk * chunk_samples)) {
+            if (!WavpackSeekSample64 (wpc, (int64_t) start_chunk * chunk_samples)) {
                 printf ("seeking_test(): seek error!\n");
                 return -1;
             }
@@ -453,7 +459,7 @@ static int seeking_test (char *filename, uint32_t test_count)
                     return -1;
                 }
 
-                format_samples (bps, (unsigned char *) decoded_samples, decoded_samples, samples * num_chans);
+                store_samples (decoded_samples, decoded_samples, qmode, bps, samples * num_chans);
 
                 // if (frandom() < 0.0001)
                 //     decoded_samples [(int) floor (samples * frandom())] ^= 1;
@@ -463,7 +469,7 @@ static int seeking_test (char *filename, uint32_t test_count)
                 MD5Final (md5_chunk, &md5_local);
 
                 if (memcmp (chunked_md5 + current_chunk * 16, md5_chunk, sizeof (md5_chunk))) {
-                    printf ("seeking_test(): seek+decode error!\n");
+                    printf ("seeking_test(): seek+decode error at %lld!\n", (long long int) current_chunk * chunk_samples);
                     return -1;
                 }
 
@@ -848,7 +854,7 @@ static int run_test (int wpconfig_flags, int test_flags, int bits, int num_chans
         }
     }
 
-    WavpackSetConfiguration (out_wpc, &wpconfig, -1);
+    WavpackSetConfiguration64 (out_wpc, &wpconfig, -1, NULL);
     WavpackPackInit (out_wpc);
 
     while (seconds < num_seconds) {
@@ -898,10 +904,7 @@ static int run_test (int wpconfig_flags, int test_flags, int bits, int num_chans
         }
 
 	WavpackPackSamples (out_wpc, (int32_t *) destin, ENCODE_SAMPLES);
-
-        if (wpconfig.bytes_per_sample != 4)
-            format_samples (wpconfig.bytes_per_sample, (unsigned char *) destin, (int32_t *) destin, ENCODE_SAMPLES * num_chans);
-
+        store_samples (destin, (int32_t *) destin, 0, wpconfig.bytes_per_sample, ENCODE_SAMPLES * num_chans);
         MD5Update (&md5_context, (unsigned char *) destin, wpconfig.bytes_per_sample * ENCODE_SAMPLES * num_chans);
 
         sequencing_angle += 2.0 * M_PI / SAMPLE_RATE / speed * ENCODE_SAMPLES;
@@ -1024,7 +1027,7 @@ static void *decode_thread (void *threadid)
         if (!samples)
             break;
 
-        format_samples (bps, (unsigned char *) decoded_samples, decoded_samples, samples * num_chans);
+        store_samples (decoded_samples, decoded_samples, 0, bps, samples * num_chans);
         MD5Update (&md5_context, (unsigned char *) decoded_samples, bps * samples * num_chans);
         wd->sample_count += samples;
     }
@@ -1389,48 +1392,203 @@ static void float_to_32bit_integer_samples (float *samples, int num_samples)
     } 
 }
 
-// Reformat samples from longs in processor's native endian mode to
-// little-endian data with (possibly) less than 4 bytes / sample.
+// Code to store samples. Source is an array of int32_t data (which is what WavPack uses
+// internally), but the destination can have from 1 to 4 bytes per sample. Also, the destination
+// data is assumed to be little-endian and signed, except for byte data which is unsigned (these
+// are WAV file defaults). The endian and signedness can be overridden with the qmode flags
+// to support other formats.
 
-static unsigned char *format_samples (int bps, unsigned char *dst, int32_t *src, uint32_t samcnt)
+static void *store_little_endian_unsigned_samples (void *dst, int32_t *src, int bps, int count);
+static void *store_little_endian_signed_samples (void *dst, int32_t *src, int bps, int count);
+static void *store_big_endian_unsigned_samples (void *dst, int32_t *src, int bps, int count);
+static void *store_big_endian_signed_samples (void *dst, int32_t *src, int bps, int count);
+
+static void *store_samples (void *dst, int32_t *src, int qmode, int bps, int count)
 {
+    if (qmode & QMODE_BIG_ENDIAN) {
+        if ((qmode & QMODE_UNSIGNED_WORDS) || (bps == 1 && !(qmode & QMODE_SIGNED_BYTES)))
+            return store_big_endian_unsigned_samples (dst, src, bps, count);
+        else
+            return store_big_endian_signed_samples (dst, src, bps, count);
+    }
+    else if ((qmode & QMODE_UNSIGNED_WORDS) || (bps == 1 && !(qmode & (QMODE_SIGNED_BYTES | QMODE_DSD_AUDIO))))
+        return store_little_endian_unsigned_samples (dst, src, bps, count);
+    else
+        return store_little_endian_signed_samples (dst, src, bps, count);
+}
+
+static void *store_little_endian_unsigned_samples (void *dst, int32_t *src, int bps, int count)
+{
+    unsigned char *dptr = dst;
     int32_t temp;
 
     switch (bps) {
 
         case 1:
-            while (samcnt--)
-                *dst++ = *src++ + 128;
+            while (count--)
+                *dptr++ = *src++ + 0x80;
 
             break;
 
         case 2:
-            while (samcnt--) {
-                *dst++ = (unsigned char) (temp = *src++);
-                *dst++ = (unsigned char) (temp >> 8);
+            while (count--) {
+                *dptr++ = (unsigned char) (temp = *src++ + 0x8000);
+                *dptr++ = (unsigned char) (temp >> 8);
             }
 
             break;
 
         case 3:
-            while (samcnt--) {
-                *dst++ = (unsigned char) (temp = *src++);
-                *dst++ = (unsigned char) (temp >> 8);
-                *dst++ = (unsigned char) (temp >> 16);
+            while (count--) {
+                *dptr++ = (unsigned char) (temp = *src++ + 0x800000);
+                *dptr++ = (unsigned char) (temp >> 8);
+                *dptr++ = (unsigned char) (temp >> 16);
             }
 
             break;
 
         case 4:
-            while (samcnt--) {
-                *dst++ = (unsigned char) (temp = *src++);
-                *dst++ = (unsigned char) (temp >> 8);
-                *dst++ = (unsigned char) (temp >> 16);
-                *dst++ = (unsigned char) (temp >> 24);
+            while (count--) {
+                *dptr++ = (unsigned char) (temp = *src++ + 0x80000000);
+                *dptr++ = (unsigned char) (temp >> 8);
+                *dptr++ = (unsigned char) (temp >> 16);
+                *dptr++ = (unsigned char) (temp >> 24);
             }
 
             break;
     }
 
-    return dst;
+    return dptr;
+}
+
+static void *store_little_endian_signed_samples (void *dst, int32_t *src, int bps, int count)
+{
+    unsigned char *dptr = dst;
+    int32_t temp;
+
+    switch (bps) {
+
+        case 1:
+            while (count--)
+                *dptr++ = *src++;
+
+            break;
+
+        case 2:
+            while (count--) {
+                *dptr++ = (unsigned char) (temp = *src++);
+                *dptr++ = (unsigned char) (temp >> 8);
+            }
+
+            break;
+
+        case 3:
+            while (count--) {
+                *dptr++ = (unsigned char) (temp = *src++);
+                *dptr++ = (unsigned char) (temp >> 8);
+                *dptr++ = (unsigned char) (temp >> 16);
+            }
+
+            break;
+
+        case 4:
+            while (count--) {
+                *dptr++ = (unsigned char) (temp = *src++);
+                *dptr++ = (unsigned char) (temp >> 8);
+                *dptr++ = (unsigned char) (temp >> 16);
+                *dptr++ = (unsigned char) (temp >> 24);
+            }
+
+            break;
+    }
+
+    return dptr;
+}
+
+static void *store_big_endian_unsigned_samples (void *dst, int32_t *src, int bps, int count)
+{
+    unsigned char *dptr = dst;
+    int32_t temp;
+
+    switch (bps) {
+
+        case 1:
+            while (count--)
+                *dptr++ = *src++ + 0x80;
+
+            break;
+
+        case 2:
+            while (count--) {
+                *dptr++ = (unsigned char) ((temp = *src++ + 0x8000) >> 8);
+                *dptr++ = (unsigned char) temp;
+            }
+
+            break;
+
+        case 3:
+            while (count--) {
+                *dptr++ = (unsigned char) ((temp = *src++ + 0x800000) >> 16);
+                *dptr++ = (unsigned char) (temp >> 8);
+                *dptr++ = (unsigned char) temp;
+            }
+
+            break;
+
+        case 4:
+            while (count--) {
+                *dptr++ = (unsigned char) ((temp = *src++ + 0x80000000) >> 24);
+                *dptr++ = (unsigned char) (temp >> 16);
+                *dptr++ = (unsigned char) (temp >> 8);
+                *dptr++ = (unsigned char) temp;
+            }
+
+            break;
+    }
+
+    return dptr;
+}
+
+static void *store_big_endian_signed_samples (void *dst, int32_t *src, int bps, int count)
+{
+    unsigned char *dptr = dst;
+    int32_t temp;
+
+    switch (bps) {
+
+        case 1:
+            while (count--)
+                *dptr++ = *src++;
+
+            break;
+
+        case 2:
+            while (count--) {
+                *dptr++ = (unsigned char) ((temp = *src++) >> 8);
+                *dptr++ = (unsigned char) temp;
+            }
+
+            break;
+
+        case 3:
+            while (count--) {
+                *dptr++ = (unsigned char) ((temp = *src++) >> 16);
+                *dptr++ = (unsigned char) (temp >> 8);
+                *dptr++ = (unsigned char) temp;
+            }
+
+            break;
+
+        case 4:
+            while (count--) {
+                *dptr++ = (unsigned char) ((temp = *src++) >> 24);
+                *dptr++ = (unsigned char) (temp >> 16);
+                *dptr++ = (unsigned char) (temp >> 8);
+                *dptr++ = (unsigned char) temp;
+            }
+
+            break;
+    }
+
+    return dptr;
 }
