@@ -560,7 +560,7 @@ static int read_new_config_info (WavpackContext *wpc, WavpackMetadata *wpmd)
 
     // if there's any data, the first two bytes are file_format and qmode flags
 
-    if (bytecnt) {
+    if (bytecnt >= 2) {
         wpc->file_format = *byteptr++;
         wpc->config.qmode = (wpc->config.qmode & ~0xff) | *byteptr++;
         bytecnt -= 2;
@@ -593,6 +593,10 @@ static int read_new_config_info (WavpackContext *wpc, WavpackMetadata *wpmd)
                         for (i = 0; i < nchans; ++i)
                             if (bytecnt) {
                                 wpc->channel_reordering [i] = *byteptr++;
+
+                                if (wpc->channel_reordering [i] >= nchans)  // make sure index is in range
+                                    wpc->channel_reordering [i] = 0;
+
                                 bytecnt--;
                             }
                             else
@@ -1053,7 +1057,7 @@ static int seek_eof_information (WavpackContext *wpc, int64_t *final_index, int
 
     // start 1MB from the end-of-file, or from the start if the file is not that big
 
-    if (reader->get_length (id) > 1048576LL)
+    if (reader->get_length (id) > (int64_t) 1048576)
         reader->set_pos_rel (id, -1048576, SEEK_END);
     else
         reader->set_pos_abs (id, 0);
@@ -1085,7 +1089,7 @@ static int seek_eof_information (WavpackContext *wpc, int64_t *final_index, int
             // beginning of the file) and try again
 
             if (!blocks) {
-                if (current_pos > 2000000LL)
+                if (current_pos > (int64_t) 2000000)
                     reader->set_pos_rel (id, -2000000, SEEK_CUR);
                 else
                     reader->set_pos_abs (id, 0);
@@ -1113,7 +1117,7 @@ static int seek_eof_information (WavpackContext *wpc, int64_t *final_index, int
             audio_blocks++;
         }
         else if (!audio_blocks) {
-            if (current_pos > 1048576LL)
+            if (current_pos > (int64_t) 1048576)
                 reader->set_pos_rel (id, -1048576, SEEK_CUR);
             else
                 reader->set_pos_abs (id, 0);
